var R = require('ramda')

function addHandlers(componentInstance, ref) {
	if ( ref === void 0 ) ref = {};
	var changeState = ref.changeState;

	var setState = componentInstance.setState.bind(componentInstance)
	var evolveState = R.pipe(
		R.evolve,
		setState
	)

	componentInstance.handleValue = R.memoize(function (stateProp) { return function (value) {
		setState(( obj = {}, obj[stateProp] = value, obj ))
		var obj;
	}; })

	componentInstance.handleEvent = R.memoize(function (stateProp, eventPath) {
		if ( eventPath === void 0 ) eventPath = ['target', 'value'];

		return function (value) {
		setState(( obj = {}, obj[stateProp] = R.path(eventPath, value), obj ))
		var obj;
	};
	})

	componentInstance.handleKeyValuePair = R.memoize(function (stateProp) { return function (key, value) {
		evolveState(( obj$1 = {}, obj$1[stateProp] = R.merge(R.__, ( obj = {}, obj[key] = value, obj )), obj$1 ))
		var obj;
		var obj$1;
	}; })

	componentInstance.handleArrayProp = R.memoize(function (stateProp, key) { return function (index, value) {
		evolveState(( obj = {}, obj[stateProp] = R.adjust(
				R.assoc(key, value),
				index
			), obj ))
		var obj;
	}; })

	// Custom changeState handlers
	R.forEach(
		R.pipe(
			R.toPairs, // Array of key-value pairs
			R.forEach(
				function (ref) {
					var methodName = ref[0];
					var handler = ref[1];

					// Assign handler method
					componentInstance[methodName] = R.pipe(
						(handler.length === 1) ? R.always(handler) : R.curry(handler),
						setState
					)
				}
			)
		),
		[].concat(changeState) // Ensure array
	)
}

module.exports = {
	addHandlers: addHandlers
}
